@@ -1,6 +1,6 @@
 /* xdaliclock - a melting digital clock
- * Copyright (c) 1991, 1992, 1993, 1994, 1995, 1996, 1997
- *  Jamie Zawinski <jwz@netscape.com>
+ * Copyright (c) 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998
+ *  Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -22,6 +22,7 @@
 
 #include <stdio.h>
 #include <ctype.h>
+#include <time.h>
 #include <X11/Intrinsic.h>
 #include <X11/IntrinsicP.h>
 #include <X11/StringDefs.h>
@@ -59,37 +60,37 @@ static void hack_version P((void));
 
 static XrmOptionDescRec options [] = {
   /* global options */
-  { "-root",		".root",		XrmoptionNoArg, "True" },
-  { "-onroot",		".root",		XrmoptionNoArg, "True" },
-  { "-window",		".root",		XrmoptionNoArg, "False" },
-  { "-fullscreen",	".fullScreen",		XrmoptionNoArg, "True" },
-  { "-mono",		".mono",		XrmoptionNoArg, "True" },
-  { "-visual",		".visualID",		XrmoptionSepArg, 0 },
-  { "-install",		".installColormap",	XrmoptionNoArg, "True" },
-  { "-title",		".title",		XrmoptionSepArg, 0  }, /* R3 */
-  { "-xrm",		0,			XrmoptionResArg, 0 },
+  { "-root",            ".root",                XrmoptionNoArg, "True" },
+  { "-onroot",          ".root",                XrmoptionNoArg, "True" },
+  { "-window",          ".root",                XrmoptionNoArg, "False" },
+  { "-fullscreen",      ".fullScreen",          XrmoptionNoArg, "True" },
+  { "-mono",            ".mono",                XrmoptionNoArg, "True" },
+  { "-visual",          ".visualID",            XrmoptionSepArg, 0 },
+  { "-install",         ".installColormap",     XrmoptionNoArg, "True" },
+  { "-title",           ".title",               XrmoptionSepArg, 0  }, /* R3 */
+  { "-xrm",             0,                      XrmoptionResArg, 0 },
 
   /* options from digital.c */
-  { "-12",		".mode",		XrmoptionNoArg, "12" },
-  { "-24",		".mode",		XrmoptionNoArg, "24" },
-  { "-datemode",	".datemode",		XrmoptionSepArg, 0 },
-  { "-mm/dd/yy",	".datemode",		XrmoptionNoArg, "MM/DD/YY" },
-  { "-dd/mm/yy",	".datemode",		XrmoptionNoArg, "DD/MM/YY" },
-  { "-seconds",		".seconds",		XrmoptionNoArg, "True" },
-  { "-noseconds",	".seconds",		XrmoptionNoArg, "False" },
-  { "-cycle",		".cycle",		XrmoptionNoArg, "True" },
-  { "-nocycle",		".cycle",		XrmoptionNoArg, "False" },
-  { "-font",		".font",		XrmoptionSepArg, 0 },
-  { "-fn",		".font",		XrmoptionSepArg, 0 },
-  { "-builtin",		".font",		XrmoptionNoArg, "BUILTIN" },
-  { "-builtin2",	".font",		XrmoptionNoArg, "BUILTIN2" },
-  { "-memory",		".memory",		XrmoptionSepArg, 0 },
-  { "-sleaze-level",	".memory",		XrmoptionSepArg, 0 },
-  { "-oink",		".memory",		XrmoptionNoArg, "Medium" },
-  { "-oink-oink",	".memory",		XrmoptionNoArg, "High" },
-  { "-transparent",	".transparent",		XrmoptionNoArg, "True" },
-  { "-nontransparent",	".transparent",		XrmoptionNoArg, "False" },
-  { "-countdown",	".countdown",		XrmoptionSepArg, 0 }
+  { "-12",              ".mode",                XrmoptionNoArg, "12" },
+  { "-24",              ".mode",                XrmoptionNoArg, "24" },
+  { "-datemode",        ".datemode",            XrmoptionSepArg, 0 },
+  { "-mm/dd/yy",        ".datemode",            XrmoptionNoArg, "MM/DD/YY" },
+  { "-dd/mm/yy",        ".datemode",            XrmoptionNoArg, "DD/MM/YY" },
+  { "-seconds",         ".seconds",             XrmoptionNoArg, "True" },
+  { "-noseconds",       ".seconds",             XrmoptionNoArg, "False" },
+  { "-cycle",           ".cycle",               XrmoptionNoArg, "True" },
+  { "-nocycle",         ".cycle",               XrmoptionNoArg, "False" },
+  { "-font",            ".font",                XrmoptionSepArg, 0 },
+  { "-fn",              ".font",                XrmoptionSepArg, 0 },
+  { "-builtin",         ".font",                XrmoptionNoArg, "BUILTIN" },
+  { "-builtin2",        ".font",                XrmoptionNoArg, "BUILTIN2" },
+  { "-memory",          ".memory",              XrmoptionSepArg, 0 },
+  { "-sleaze-level",    ".memory",              XrmoptionSepArg, 0 },
+  { "-oink",            ".memory",              XrmoptionNoArg, "Medium" },
+  { "-oink-oink",       ".memory",              XrmoptionNoArg, "High" },
+  { "-transparent",     ".transparent",         XrmoptionNoArg, "True" },
+  { "-nontransparent",  ".transparent",         XrmoptionNoArg, "False" },
+  { "-countdown",       ".countdown",           XrmoptionSepArg, 0 }
 };
 
 int
@@ -97,8 +98,8 @@ int
 main (int argc, char **argv)
 #else  /* !__STDC__ */
 main (argc, argv)
-	int argc;
-	char **argv;
+        int argc;
+        char **argv;
 #endif /* !__STDC__ */
 {
   Widget toplevel = 0;
@@ -128,8 +129,8 @@ main (argc, argv)
     app = XtCreateApplicationContext ();
     XtAppSetFallbackResources (app, defaults);
     dpy = XtOpenDisplay (app, 0, 0, progclass,
-			 options, sizeof (options) / sizeof (options [0]),
-			 &argc, argv);
+                         options, sizeof (options) / sizeof (options [0]),
+                         &argc, argv);
     if (!dpy) exit (-1);
     XtGetApplicationNameAndClass (dpy, &progname, &progclass);
     db = XtDatabase (dpy);
@@ -152,11 +153,11 @@ main (argc, argv)
 
     argv_db = 0;
     XrmParseCommand (&argv_db,
-		     options, sizeof (options) / sizeof (options [0]),
-		     progname, &argc, argv);
+                     options, sizeof (options) / sizeof (options [0]),
+                     progname, &argc, argv);
     toplevel = XtInitialize (progname, progclass, options,
-			     sizeof (options) / sizeof (options [0]),
-			     &argc, argv);
+                             sizeof (options) / sizeof (options [0]),
+                             &argc, argv);
     dpy = XtDisplay (toplevel);
     server_db = XtDatabase (dpy);
     for (i = 0, j = 0; defaults [i]; i++)
@@ -164,10 +165,10 @@ main (argc, argv)
     tmp = (char *) malloc (j + 1);
     for (i = 0; defaults [i]; i++)
       {
-	tmp = strcat (tmp, defaults [i]);
-	tmp += strlen (tmp);
-	*tmp++ = '\n';
-	*tmp = 0;
+        tmp = strcat (tmp, defaults [i]);
+        tmp += strlen (tmp);
+        *tmp++ = '\n';
+        *tmp = 0;
       }
     tmp -= j;
     fallback_db = XrmGetStringDatabase (tmp);
@@ -187,8 +188,8 @@ main (argc, argv)
     {
       int help_p = !strcmp (argv [1], "-help");
       if (!help_p)
-	fprintf (stderr, "%s: unrecognised option \"%s\"\n",
-		 progname, argv[1]);
+        fprintf (stderr, "%s: unrecognised option \"%s\"\n",
+                 progname, argv[1]);
       usage ();
       exit (help_p ? 0 : -1);
     }
@@ -202,8 +203,69 @@ main (argc, argv)
     char *s = get_string_resource ("countdown", "Countdown");
     if (s && *s)
       {
-	/* #### write date parser here (for the thousandth time) */
-	countdown = atol(s);
+        long l = 0;
+        char c = 0;
+        char mon[255];
+        int day = 0;
+        int hour = 0;
+        int min = 0;
+        int sec = 0;
+        int year = 0;
+        *mon = 0;
+        if (1 == sscanf (s, " %lu %c", &l, &c))
+          countdown = l;
+        else if (6 == sscanf (s, " %s %d %d:%d:%d %d %c",
+                              &mon, &day, &hour, &min, &sec, &year, &c))
+          {
+            struct tm tm;
+            int imonth = -1;
+            const char *months[] = {"jan", "feb", "mar", "apr", "may", "jun",
+                                    "jul", "aug", "sep", "oct", "nov", "dec" };
+
+            for (l = 0; mon[l]; l++)
+              if (mon[l] >= 'A' && mon[l] <= 'Z')
+                mon[l] = mon[l] + ('a'-'A');
+
+            for (l = 0; l < 12; l++)
+              if (!strcmp(months[l], mon))
+                {
+                  imonth = l;
+                  break;
+                }
+            if (imonth < 0)
+              {
+                fprintf (stderr, "%s: unparsable month: \"%s\"\n",
+                         progname, mon);
+                goto LOSE2;
+              }
+            if (year > 1900) year -= 1900;
+            tm.tm_sec = sec;
+            tm.tm_min = min;
+            tm.tm_hour = hour;
+            tm.tm_mday = day;
+            tm.tm_mon = imonth;
+            tm.tm_year = year;
+            tm.tm_wday = 0;     /* ignored */
+            tm.tm_yday = 0;     /* ignored */
+            tm.tm_isdst = -1;
+            countdown = mktime (&tm);
+            if ((long) countdown < 0L)
+              goto LOSE1;
+          }
+        else
+          {
+          LOSE1:
+            fprintf (stderr, "%s: unparsable date: \"%s\"\n", progname, s);
+          LOSE2:
+            fprintf (stderr,
+          "%s: argument to -countdown should be a time_t integer\n"
+        "\t (number of seconds past \"Jan 1 00:00:00 GMT 1970\");\n"
+        "\t Or, a string of the form \"Mmm DD HH:MM:SS YYYY\",\n"
+        "\t for example, \"Jan 1 00:00:00 2000\".\n"
+        "\t This string is interpreted in the local time zone.\n",
+                     progname);
+            exit (1);
+          }
       }
     if (s) free(s);
   }
@@ -219,25 +281,25 @@ main (argc, argv)
     {
       do_shape = get_boolean_resource ("transparent", "Transparent");
       if (do_shape)
-	{
-	  Visual *v = get_overlay_visual (screen, &overlay_transparent_pixel);
-	  if (v)
-	    {
-	      do_shape = 0;
-	      do_overlay = 1;
-	      visual = v;
-	    }
-	}
+        {
+          Visual *v = get_overlay_visual (screen, &overlay_transparent_pixel);
+          if (v)
+            {
+              do_shape = 0;
+              do_overlay = 1;
+              visual = v;
+            }
+        }
 
       if (!do_overlay)
-	visual = get_visual_resource (screen, "visualID", "VisualID", True);
+        visual = get_visual_resource (screen, "visualID", "VisualID", True);
 
       if ((visual != DefaultVisualOfScreen (screen)) ||
-	  get_boolean_resource ("installColormap", "InstallColormap"))
-	cmap = XCreateColormap (dpy, RootWindowOfScreen (screen),
-				visual, AllocNone);
+          get_boolean_resource ("installColormap", "InstallColormap"))
+        cmap = XCreateColormap (dpy, RootWindowOfScreen (screen),
+                                visual, AllocNone);
       else
-	cmap = DefaultColormapOfScreen (screen);
+        cmap = DefaultColormapOfScreen (screen);
     }
 
   wander_p = (root_p || full_screen_p);
@@ -248,7 +310,7 @@ main (argc, argv)
     {
       /* `window' has already been set, above. */
       if (toplevel)
-	XtDestroyWidget (toplevel);
+        XtDestroyWidget (toplevel);
     }
   else
     {
@@ -272,8 +334,8 @@ main (argc, argv)
 
 #if (XtSpecificationRelease >= 4)
       toplevel = XtAppCreateShell (progname, progclass,
-				   applicationShellWidgetClass, dpy,
-				   av, ac);
+                                   applicationShellWidgetClass, dpy,
+                                   av, ac);
 #else /* R3 */
       /* This won't work with non-default visuals; upgrade!! */
       XtSetValues (toplevel, av, ac);
@@ -286,27 +348,27 @@ main (argc, argv)
       XtGetValues (toplevel, av, ac);
 
       if (full_screen_p)
-	{
-	  w = WidthOfScreen  (screen);
-	  h = HeightOfScreen (screen);
-	}
+        {
+          w = WidthOfScreen  (screen);
+          h = HeightOfScreen (screen);
+        }
       if (ow <= 1 || oh <= 1)
-	{
-	  Dimension ww = w + xmargin + xmargin;
-	  Dimension hh = h + ymargin + ymargin;
-	  ac = 0;
-	  XtSetArg (av [ac], XtNwidth,  ww); ac++;
-	  XtSetArg (av [ac], XtNheight, hh); ac++;
-	  XtSetValues (toplevel, av, ac);
-	}
+        {
+          Dimension ww = w + xmargin + xmargin;
+          Dimension hh = h + ymargin + ymargin;
+          ac = 0;
+          XtSetArg (av [ac], XtNwidth,  ww); ac++;
+          XtSetArg (av [ac], XtNheight, hh); ac++;
+          XtSetValues (toplevel, av, ac);
+        }
       if (full_screen_p)
-	{
-	  /* #### Want this to set UPosition, not PPosition... */
-	  ac = 0;
-	  XtSetArg (av [ac], XtNx, -xmargin); ac++;
-	  XtSetArg (av [ac], XtNy, -ymargin); ac++;
-	  XtSetValues (toplevel, av, ac);
-	}
+        {
+          /* #### Want this to set UPosition, not PPosition... */
+          ac = 0;
+          XtSetArg (av [ac], XtNx, -xmargin); ac++;
+          XtSetArg (av [ac], XtNy, -ymargin); ac++;
+          XtSetValues (toplevel, av, ac);
+        }
 
       XtRealizeWidget (toplevel);
       window = XtWindow (toplevel);
@@ -315,9 +377,9 @@ main (argc, argv)
 #if (XtSpecificationRelease >= 4)
   if (!root_p)
     XSetStandardProperties(dpy, window, progname, progname,
-			   0, /* icon pixmap */
-			   argv_copy, argc_copy,
-			   NULL /* XSizeHints* */);
+                           0, /* icon pixmap */
+                           argv_copy, argc_copy,
+                           NULL /* XSizeHints* */);
   free(argv_copy);
 #endif
 
@@ -330,7 +392,7 @@ static void
 usage P((void))
 {
   fprintf (stderr, "\n%s\n\
-                  http://people.netscape.com/jwz/xdaliclock/\n\n\
+                  http://www.jwz.org/xdaliclock/\n\n\
 usage: %s [ options ]\n\
 where options include\n\
 \n\
@@ -389,7 +451,7 @@ where options include\n\
   -nontransparent               Don't.\n\n");
 #if !defined(BUILTIN_FONT) && !defined(BUILTIN_FONT_2)
   fprintf(stderr,
-	  "This version has been compiled without the builtin fonts.\n\n");
+          "This version has been compiled without the builtin fonts.\n\n");
 #endif
 }
 
@@ -404,16 +466,16 @@ hack_version P((void))
   while (*src)
     if (!strncmp ("Copyright (c)", src, 13))
       {
-	*dst1++ = '\251';
-	for (i = 0; i < 13; i++) *dst2++ = *src++;
+        *dst1++ = '\251';
+        for (i = 0; i < 13; i++) *dst2++ = *src++;
       }
     else if (*src == '(') *dst1++ = *dst2++ = '<', src++;
     else if (*src == ')') *dst1++ = *dst2++ = '>', src++;
     else if (digitp (src[0]) && digitp (src[1]) && digitp (src[2]) &&
-	     digitp (src[3]) && src[4] == ',')
+             digitp (src[3]) && src[4] == ',')
       {
-	src += 5;
-	if (*src == ' ') src++;
+        src += 5;
+        if (*src == ' ') src++;
       }
     else
       *dst1++ = *dst2++ = *src++;
@@ -434,7 +496,7 @@ strcasecmp (s1, s2)
     return 1;
   for (i = 0; i < L1; i++)
     if ((isupper (s1[i]) ? _tolower (s1[i]) : s1[i]) !=
-	(isupper (s2[i]) ? _tolower (s2[i]) : s2[i]))
+        (isupper (s2[i]) ? _tolower (s2[i]) : s2[i]))
       return 1;
   return 0;
 }
